import os, sys
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
import matplotlib as mpl
import argparse

IS_GUI = os.environ.get("GENOVIS_GUI") == "1"
def fail(msg):
    if IS_GUI:
        raise RuntimeError(msg)
    else:
        print(f"[ERROR] {msg}", file=sys.stderr, flush=True)
        sys.exit(1)

def add_subparser(subparsers):
    parser = subparsers.add_parser("rohpainter",help="Plot runs of homozygous regions barplot.")
    parser.add_argument('--d',help='Dataframe(1st column:population label, 2nd: Individual ID, 3rd: chromosome, 4th: start position, 5th: end position)',type=str,required=True)
    parser.add_argument('--i',help='Genome index(1st column:chromosome, 2nd: size)',type=str,required=True)
    parser.add_argument('--mode',help='Output mode: "int" for interactive show, solid format to save as pdf,svg or png',type=str,choices=['int', 'solid'], default='int')
    parser.add_argument('--x',help='Horizontal size of figure',default=10,type=float)
    parser.add_argument('--y',help='Vertical size of figure',default=4,type=float)
    parser.add_argument('--yt',help='Font size of yticks',default=9,type=float)
    parser.add_argument('--xt',help='Font size of xticks',default=9,type=float)
    parser.add_argument('--t',help='Threshold for identifying common intervals(e.g., 0.8). Please use “--t false”, if you do not want to pinpoint common intervals.',default="false",type=str)
    parser.add_argument('--tc',help='Threshold line color',default="red",type=str)
    parser.add_argument('--tw',help='Threshold line width',default=0.75,type=float)
    parser.add_argument('--sl', help='Show labels in y axis', choices=['true', 'false'], default='true', type=str)
    parser.add_argument('--c',help='Choosing colormap (please see https://matplotlib.org/stable/users/explain/colors/colormaps.html)', default="Paired",type=str)
    parser.add_argument('--Chr',help='Chromosomal prefix (chr, chromosome, contig, etc.)',type=str,default="Chromosome")
    parser.add_argument('--f',help='Font family',default='Calibri',type=str)
    parser.add_argument('--fs',help='Font size',default=12,type=float)
    parser.add_argument('--o',help='Output file prefix',default="out",type=str)
    parser.add_argument('--ft',help='Format type pdf, tif, tiff, jpg, jpeg, eps, pgf, png, ps, raw, rgba, svgz, svg or webp',choices=['pdf','svg','svgz','png','tif','tiff','jpg','jpeg','eps','pgf','ps','raw','rgba','webp'],type=str,default='jpg')
    parser.add_argument('--dpi',help='dpi',type=int,default=300)
    parser.set_defaults(func=run)

def run(args):
    t_raw = str(args.t).strip().lower()
    if t_raw == "false":
        use_threshold = False
        thresh_frac = None
    else:
        try:
            thresh_frac = float(t_raw)
        except ValueError:
            fail("--t must be 'false' or a number between 0 and 1 (e.g., 0.8).")
        if not (0.0 < thresh_frac <= 1.0):
            fail("--t must be between 0 and 1 (e.g., 0.8).")
        use_threshold = True
    if not os.path.isfile(args.d): fail(f"Dataframe not found: {args.d}") 
    if not os.path.isfile(args.i): fail(f"Genome index not found: {args.i}") 
    if args.x <= 0 or args.y <= 0: fail("--x/--y (figure width/height) must be > 0") 
    if args.xt <= 0 or args.yt <= 0 : fail("Font size of x/y ticks must be > 0") 
    if args.tw <= 0: fail("--tw (Threshold line width) must be > 0")
    if args.fs <= 0: fail("--fs (font size) must be > 0")
    if args.dpi < 300: fail("--dpi must be >= 300")
    print("===== Dataframe preparation ... =====")
    df = pd.read_csv(args.d,sep=r'\s+',header=None,names=['pop', 'ind', 'chr', 'start', 'end'],dtype={'pop': str, 'ind': str, 'chr': str, 'start': int, 'end': int})
    print("Dataframe:")
    print(df)
    index = pd.read_csv(args.i,sep=r'\s+',header=None,names=['chr', 'chr_size'],dtype={'chr': str, 'chr_size': int})
    print("Genome index:")
    print(index)
    df2 = pd.merge(df,index,on='chr')
    if df2.empty:fail("After merging input dataframe with genome index, no chromosomes matched. ""Check that chromosome names in --d and --i use the same naming convention.")
    individuals = df2['ind'].unique()
    n_inds = len(individuals)
    y_positions = {ind: i for i, ind in enumerate(individuals)}
    pops = df2['pop'].unique()
    n = pops.shape[0]
    color = sns.color_palette(args.c, n)
    colors = dict(zip(pops, color))
    sns.set_theme(style="ticks", rc={"axes.grid": False})
    shared_list = []
    mpl.rcParams['font.family'] = args.f
    mpl.rcParams['font.size'] = args.fs
    mpl.rcParams['axes.titlesize']   = args.fs
    mpl.rcParams['axes.labelsize']   = args.fs
    mpl.rcParams['legend.fontsize']  = args.fs
    mpl.rcParams['ytick.labelsize'] = args.yt
    mpl.rcParams['xtick.labelsize'] = args.xt
    mpl.rcParams['font.weight']       = 'bold'
    for chrom, sub in df2.groupby('chr'):
        size_bp = sub['chr_size'].iloc[0]
        if use_threshold:
            threshold = thresh_frac*n_inds
            breakpoints = sorted(set(sub['start'].tolist() + sub['end'].tolist()))
            shared_intervals = []
            for i in range(len(breakpoints) - 1):
                seg_start = breakpoints[i]
                seg_end = breakpoints[i + 1]
                count_cover = sub[(sub['start']<= seg_start)&(sub['end'] >= seg_end)]['ind'].nunique()
                if count_cover >= threshold:
                    if not shared_intervals or seg_start > shared_intervals[-1][1]:
                        shared_intervals.append([seg_start, seg_end])
                    else:
                        shared_intervals[-1][1] = seg_end
        else:
            shared_intervals = []

        for (s_bp, e_bp) in shared_intervals:shared_list.append({'chr': chrom,'start': s_bp,'end': e_bp})
        fig, ax = plt.subplots(figsize=(args.x, args.y))
        ax.set_title(f'{args.Chr} {chrom}')
        ax.set_xlim(0, size_bp / 1e6)
        if args.sl == 'true':
            ax.set_yticks(list(y_positions.values()))
            ax.set_yticklabels(list(y_positions.keys()), fontsize=args.yt)
        if args.sl == 'false':
            ax.set_yticks([])
        ax.set_ylim(-1, len(individuals))
        ax.set_xlabel('Position (Mbp)')
        ax.set_ylabel('Individual')
        for _, row in sub.iterrows():
            y = y_positions[row['ind']] - 0.3
            start_mbp = row['start'] / 1e6
            width_mbp = (row['end'] - row['start']) / 1e6
            ax.broken_barh([(start_mbp, width_mbp)], (y, 0.6), facecolors=colors[row['pop']])
        if use_threshold:
            for (s_bp, e_bp) in shared_intervals:
                s_mbp = s_bp / 1e6
                e_mbp = e_bp / 1e6
                ax.axvline(x=s_mbp,color=args.tc,linestyle='--',linewidth=args.tw)
                ax.axvline(x=e_mbp,color=args.tc,linestyle='--',linewidth=args.tw)

        handles = [plt.Line2D([0], [0], color=colors[p], lw=6) for p in pops]
        ax.legend(handles,pops,title='Population',loc='upper left',title_fontsize=args.fs+1,bbox_to_anchor=(1.02, 1.0),borderaxespad=0.)
        fig.subplots_adjust(right=0.75)
        plt.tight_layout()
        if args.mode == 'int':
            plt.show()
        else:
            out_fname=f"{args.o}.chr{chrom}.{args.ft}"
            fig.savefig(out_fname, dpi=args.dpi)
            plt.close(fig)
            print(f"Plot saved as {out_fname} with dpi={args.dpi}")

    if use_threshold:
        if shared_list:
            shared_df = pd.DataFrame(shared_list)
            intervals = f"{args.o}_shared_intervals.txt"
            shared_df.to_csv(intervals, sep='\t', index=False)
            print(f"Shared intervals saved to {intervals}")
        else:
            print(f"No common intervals found based on provided threshold {args.t}")


import argparse
import sys
from . import pca3d,mapden,admix,relmap,rohpainter,manplot

def build_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        prog="genovis",
        description="GENOVIS: A Python package for the visualization of population genetic analyses"
    )
    subparsers = parser.add_subparsers(title="subcommands", dest="command", metavar="<tool>",required=True)
    mapden.add_subparser(subparsers)
    pca3d.add_subparser(subparsers)
    admix.add_subparser(subparsers)
    relmap.add_subparser(subparsers)
    rohpainter.add_subparser(subparsers)
    manplot.add_subparser(subparsers)
    return parser

def main(argv=None):
    print("-----------------------------------------------------------------")
    print("-----------------------------------------------------------------")
    print("|                                                               |")
    print("|     ██████╗ ███████╗███╗   ██╗ ██████╗ ██╗   ██╗██╗███████╗   |")
    print("|    ██╔════╝ ██╔════╝████╗  ██║██╔═══██╗██║   ██║██║██╔════╝   |")
    print("|    ██║  ███╗█████╗  ██╔██╗ ██║██║   ██║██║   ██║██║███████╗   |")
    print("|    ██║   ██║██╔══╝  ██║╚██╗██║██║   ██║╚██╗ ██╔╝██║╚════██║   |")
    print("|    ╚██████╔╝███████╗██║ ╚████║╚██████╔╝ ╚████╔╝ ██║███████║   |")
    print("|     ╚═════╝ ╚══════╝╚═╝  ╚═══╝ ╚═════╝   ╚═══╝  ╚═╝╚══════╝   |")
    print("|                                                               |")
    print("-----------------------------------------------------------------")
    print("-----------------------------------------------------------------")
    print("|                   GENOVIS Version 1.0.3                       |")
    print("|  Developers: Siavash Salek Ardestani & Elmira Mohandesan      |")
    print("|  Contact: siasia6650@gmail.com                                |")
    print("|  Released in 2025, GENOVIS is a visualization toolkit         |")
    print("|  for population genomic analyses. It supports the             |")
    print("|  generation of: Manhattan plots, three dimensional PCA,       |")
    print("|  SNP density maps, admixture plots, runs of homozygosity      |")
    print("|  (ROH) intervals, and relationship matrices.                  |")
    print("-----------------------------------------------------------------")
    print("-----------------------------------------------------------------")

    parser = build_parser()
    args = parser.parse_args(argv)

    if not hasattr(args, "func"):
        parser.print_help()
        return 2
    try:
        return args.func(args)
    except KeyboardInterrupt:
        return 130
    except Exception as e:
        print(f"[ERROR] {e}", file=sys.stderr, flush=True)
        return 1

if __name__ == "__main__":
    sys.exit(main())



import os
import sys
from PyQt5.QtWidgets import QApplication, QMainWindow
from .genovis_gui import Ui_GENOVIS_GUI
class MainWindow(QMainWindow, Ui_GENOVIS_GUI):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setupUi(self)
def main():
    os.environ["GENOVIS_GUI"] = "1"
    app = QApplication(sys.argv)
    win = MainWindow()
    win.show()
    sys.exit(app.exec_())
if __name__ == "__main__":
    main()


import os, sys
import pandas as pd
import matplotlib as mpl
import matplotlib.pyplot as plt
import seaborn as sns
import argparse

IS_GUI = os.environ.get("GENOVIS_GUI") == "1"
def fail(msg):
    if IS_GUI:
        raise RuntimeError(msg)
    else:
        print(f"[ERROR] {msg}", file=sys.stderr, flush=True)
        sys.exit(1)

def add_subparser(subparsers):
    parser = subparsers.add_parser("manplot",help="Plot manhattan plot.")
    parser.add_argument('--d',help='Dataframe(1th column:chromosome, 2th column:position, 3th column:value)',type=str,required=True)
    parser.add_argument('--mode',help='Output mode: "int" for interactive show, solid format to save as pdf,svg or png',type=str,choices=['int','solid'],default='int')
    parser.add_argument('--c',help='Choosing colormaps (please see https://matplotlib.org/stable/users/explain/colors/colormaps.html)',default="plasma",type=str)
    parser.add_argument('--nc',help='Number of colors',default=10,type=int)
    parser.add_argument('--a',help='The alpha blending value, between 0 (transparent) and 1 (opaque)',default=1,type=float)
    parser.add_argument('--x',help='Horizontal size of figure',default=8.4,type=float)
    parser.add_argument('--y',help='Vertical size of figure',default=4,type=float)
    parser.add_argument('--s',help='Scatter size',default=0.5,type=float)
    parser.add_argument('--sug1',help='Suggestive line 1',default=None,type=float)
    parser.add_argument('--sug2',help='Suggestive line 2',default=None,type=float)
    parser.add_argument('--sug1lw',help='Width size of suggestive line 1 ',default=None,type=float)
    parser.add_argument('--sug2lw',help='Width size of suggestive line 2',default=None,type=float)
    parser.add_argument('--sug1c',help='Color of suggestive line 1',default="blue",type=str)
    parser.add_argument('--sug2c',help='Color of suggestive line 2',default="red",type=str)
    parser.add_argument('--xlab',help='X label',default="Chromosome",type=str)
    parser.add_argument('--ylab',help='Y label',default="Value",type=str)
    parser.add_argument('--f',help='Font family',default='Calibri',type=str)
    parser.add_argument('--fs',help='Font size',default=14,type=float)
    parser.add_argument('--xt',help='Font size of xticks',default=9,type=float)
    parser.add_argument('--yt',help='Font size of yticks',default=9,type=float)
    parser.add_argument('--o',help='Output file prefix',default="manplot_out",type=str)
    parser.add_argument('--ft',help='Format type pdf, tif, tiff, jpg, jpeg, eps, pgf, png, ps, raw, rgba, svgz, svg or webp',choices=['pdf','svg','svgz','png','tif','tiff','jpg','jpeg','eps','pgf','ps','raw','rgba','webp'],type=str,default='png')
    parser.add_argument('--dpi', help='Pixel density', type=int, default=300)
    parser.set_defaults(func=run)

def run(args):
    if not os.path.isfile(args.d): fail(f"Dataframe not found: {args.d}") 
    if args.x <= 0 or args.y <= 0: fail("--x/--y (figure width/height) must be > 0")
    if args.s <= 0: fail("--s (scatter size) must be > 0")
    if args.fs <= 0: fail("--fs (font size) must be > 0")
    if args.xt <= 0 or args.yt <= 0 : fail("Font size of x/y ticks must be > 0")
    if args.dpi < 300: fail("--dpi must be >= 300")
    if args.nc <= 0:fail("--nc (number of colors) must be > 0")
    if not (0 <= args.a <= 1):fail("--a (alpha) must be between 0 and 1")
    print("Dataframe preparation ...")
    cols = ["chr", 'pos','value']
    df = pd.read_csv(args.d,header=None,sep=r'\s+')
    if df.shape[1] != 3:fail(f"Input must have exactly 3 columns, got {df.shape[1]}.")
    df.columns = cols
    print(df)
    try:chromosomes = sorted(df['chr'].unique(), key=lambda x: float(x))
    except Exception:chromosomes = sorted(df['chr'].unique())
    chr_to_idx = {c:i for i,c in enumerate(chromosomes, start=1)}
    df['chr_idx'] = df['chr'].map(chr_to_idx)
    chr_max = df.groupby('chr')['pos'].max().reindex(chromosomes)
    total_data = chr_max.sum()
    fraction_of_gaps = 0.20
    num_chr = len(chromosomes)
    if num_chr > 1: 
        num_gaps = num_chr - 1
        gap = total_data * fraction_of_gaps / num_gaps
    else:
        gap = 0
    offset_series = (chr_max + gap).cumsum().shift(fill_value=0)
    offset = offset_series.to_dict()
    df['cum_pos'] = df['pos'] + df['chr'].map(offset)
    print("Plot preparation ...")
    mpl.rcParams['font.family'] = args.f
    mpl.rcParams['font.size']   = args.fs
    mpl.rcParams['xtick.labelsize'] = args.xt
    mpl.rcParams['ytick.labelsize'] = args.yt
    mpl.rcParams['mathtext.fontset'] = 'custom'
    mpl.rcParams['mathtext.rm']      = args.f
    mpl.rcParams['mathtext.it']      = args.f
    mpl.rcParams['mathtext.bf']      = args.f
    palette = sns.color_palette(args.c,args.nc)
    sns.set_theme(style="ticks", rc={"axes.grid": False})
    chr_colors = {c: palette[(i-1) % len(palette)] for c, i in chr_to_idx.items()}
    df['color'] = df['chr'].map(chr_colors)
    fig, ax = plt.subplots(figsize=(args.x,args.y))
    ax.scatter( df['cum_pos'], df['value'],c=df['color'], s=args.s,alpha=args.a)
    ax.spines['right'].set_visible(False)
    ax.spines['top'].set_visible(False)
    if args.sug1 is not None:ax.axhline(y=args.sug1, color=args.sug1c, linestyle='--', linewidth=args.sug1lw)
    if args.sug2 is not None:ax.axhline(y=args.sug2, color=args.sug2c, linestyle='--', linewidth=args.sug2lw)
    ticks = [offset[c] + chr_max[c]/2 for c in chromosomes]
    ax.set_xticks(ticks)
    ax.set_xticklabels(chromosomes, rotation=45)
    ax.set_xlabel(args.xlab,fontfamily=args.f, fontsize=args.fs,fontweight='bold')
    ax.set_ylabel(args.ylab,fontfamily=args.f, fontsize=args.fs,fontweight='bold')
    plt.xticks(fontsize=args.xt,fontfamily=args.f,fontweight='bold')
    plt.yticks(fontsize=args.yt,fontfamily=args.f,fontweight='bold')
    plt.tight_layout()
    if args.mode == 'int':
            plt.show()
    else:
        path = f"{args.o}.{args.ft}"
        plt.savefig(path, format=args.ft, dpi=args.dpi)
        plt.close('all')
        print(f"Plot saved as {path} with dpi={args.dpi}")


import os, sys
import pandas as pd
import matplotlib as mpl
import matplotlib.pyplot as plt
import seaborn as sns
import argparse
IS_GUI = os.environ.get("GENOVIS_GUI") == "1"
def fail(msg):
    if IS_GUI:
        raise RuntimeError(msg)
    else:
        print(f"[ERROR] {msg}", file=sys.stderr, flush=True)
        sys.exit(1)
    
def add_subparser(subparsers):
    parser = subparsers.add_parser("admix",help="Plot admixture barplot.")
    parser.add_argument('--d',help='Input dataframe (1th column:population lable, 2th column: Individual IDs, 3th column:K1 to Kn)',type=str,required=True)
    parser.add_argument('--mode',help='Output mode: "int" for interactive show, solid format to save as pdf,svg or png',type=str,choices=['int','solid'],default='int')
    parser.add_argument('--c',help='Colormap (string) user wants to use (please see https://matplotlib.org/stable/users/explain/colors/colormaps.html, default: hsv)',default="hsv",type=str)
    parser.add_argument('--sl',help='Show individual lables',choices=['true','false'],default='false',type=str)
    parser.add_argument('--x',help='Width size of figure',default=8.4,type=float)
    parser.add_argument('--y',help='Height size of figure',default=4,type=float)
    parser.add_argument('--f',help='Font family of figure',default='Calibri',type=str)
    parser.add_argument('--fs',help='Font size',default=14,type=float)
    parser.add_argument('--lws',help='Size of separator lines',default=0.75,type=float)
    parser.add_argument('--xt',help='Font size of individual labels',default=9,type=float)
    parser.add_argument('--o',help='Output file prefix',default="admix_plot",type=str)
    parser.add_argument('--ft',help='Format type pdf, tif, tiff, jpg, jpeg, eps, pgf, png, ps, raw, rgba, svgz, svg or webp',choices=['pdf','svg','svgz','png','tif','tiff','jpg','jpeg','eps','pgf','ps','raw','rgba','webp'],type=str,default='png')
    parser.add_argument('--dpi', help='dpi', type=int, default=300)
    parser.set_defaults(func=run)

def run(args):
    if not os.path.isfile(args.d): fail(f"Dataframe not found: {args.d}")
    if args.x <= 0 or args.y <= 0: fail("--x/--y (figure width/height) must be > 0")
    if args.lws <= 0: fail("--lws (Size of separator lines) must be > 0")
    if args.fs <= 0: fail("--fs (font size) must be > 0")
    if args.xt <= 0: fail("--xt (Font size of individual labels) must be > 0")
    if args.dpi < 300: fail("--dpi must be >= 300")
    df = pd.read_csv(args.d,sep=r'\s+',header=None)
    n = df.shape[1] - 2
    if n < 2: fail("The number of provided columns is not enough! Number of columns must be higher than three")
    k = df.iloc[:, 2:].apply(pd.to_numeric, errors='coerce')
    if k.isna().any().any():fail("All K columns (from 3rd onward) must be numeric (no NaN/non-numeric).")
    if (k < 0).any().any() or (k > 1).any().any():fail("All K values must be within [0, 1].")
    row_sums = k.sum(axis=1)
    if (row_sums > 1).any() or (row_sums < 0.99999).any():
        near_one = row_sums.between(0.99999, 1.00001)
        if near_one.all():k = k.div(row_sums, axis=0)
        else:fail("Sum of K components per row must be in [0.99999, 1]. Check scale (0..1 vs 0..100), extra columns, rounding, or parsing.")
    Ks = [f'K{i+1}' for i in range(n)]
    df.columns = ["Pop","ID"]+ Ks
    df["likely_assignment"] = df[Ks].idxmax(axis=1)
    df["assignment_prob"]  = df[Ks].max(axis=1)
    df_sorted = df.sort_values(by=["Pop", "assignment_prob"],ascending=[True, False]).reset_index(drop=True)
    tbl = (df_sorted.melt(id_vars=["Pop","ID","likely_assignment","assignment_prob"],value_vars=Ks,var_name="pop",value_name="prob").reset_index(drop=True))
    tbl = tbl.sort_values(by=["Pop","likely_assignment"],ascending=[True, False]).reset_index(drop=True)
    print(tbl)
    order = tbl["ID"].unique().tolist()
    order_df = pd.DataFrame({"ID": order})
    df = df.iloc[:, :-2]
    df2 = order_df.merge(df, on="ID", how="left")
    group_sizes = df2["Pop"].value_counts().loc[df2["Pop"].unique()].cumsum()
    boundaries = group_sizes[:-1]
    pops = df2["Pop"].unique()
    counts = df2["Pop"].value_counts().reindex(pops)
    cum = counts.cumsum()
    start = cum.shift(fill_value=0)
    mid = start + counts/2
    print("Plot prepration...")
    sns.set_theme(style="whitegrid")
    mpl.rcParams['font.family']=args.f
    mpl.rcParams['font.size']=args.fs
    mpl.rcParams['xtick.labelsize']=args.xt
    mpl.rcParams['mathtext.rm']=args.f
    mpl.rcParams['mathtext.it']=args.f
    mpl.rcParams['mathtext.bf']=args.f
    colors = sns.color_palette(args.c, n)
    fig, ax = plt.subplots(figsize=(args.x, args.y))
    df2[Ks].plot(kind="bar",stacked=True,color=colors,width=1,edgecolor="white",linewidth=0,ax=ax,legend=False,alpha=1)
    if args.sl.lower() == 'true':
        ax.set_xticks(range(len(df2)))
        ax.set_xticklabels(df2["ID"], rotation=90, ha='center', fontsize=args.xt)
        ax.tick_params(left=False, labelleft=False)
    elif args.sl.lower() == 'false':
        ax.set_xticks([])
        ax.set_xticklabels([])
    ax.set_yticks([])
    ax.set_yticklabels([])
    for b in boundaries:
        ax.axvline(x=b-0.5,color="black",linestyle="--",linewidth=args.lws,zorder=2,clip_on=False)
    for pop, m in zip(pops, mid):
        ax.text(x=m - 0.5,y=1.1,s=pop,ha="center",va="top",transform=ax.get_xaxis_transform(),fontsize=args.fs)
    ax.grid(False)
    sns.despine(ax=ax, top=True, right=True, left=True, bottom=True)
    assignment = f"{args.o}_likely_assignment.txt"
    tbl1 = tbl.sort_values(by=["Pop","ID"],ascending=[True, False]).reset_index(drop=True)
    tbl1.to_csv(assignment, sep='\t', index=False)
    if args.mode == 'int':
        plt.show()
    else:
        path = f"{args.o}.{args.ft}"
        plt.savefig(f"{args.o}.{args.ft}", dpi=args.dpi)
        plt.close('all')
        print(f"Plot saved as {path} with dpi={args.dpi}")

